local initialized = false;

local dataobj = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("Pathfinder", {
	type = "data source", 
	text = "Nothing to track!", 
	icon = 237385,
	OnClick = function(clickedframe, button)
		if button == "LeftButton" then
			ToggleAchievementFrame();
		elseif button == "RightButton" then
			Pathfinder.Config.ShowCompleted = not Pathfinder.Config.ShowCompleted;
		end
		Pathfinder.Update();
	end,
})

local function Initialize()
	if not Pathfinder.Config then
		Pathfinder.Config = {
			ShowCompleted = false,
		}
	end

	Pathfinder.Status = {
		[11446] = { -- Broekn Isles Pathfinder, Part Two
			Completed = false,
			Spacing = "",
			[11190] = { --Broken Isles Pathfinder, Part One
				Completed = false,
				Spacing = "   ",
				[10672] = { -- Broken Isles Diplomat
					Completed = false,
					Spacing = "      ",
				}, 
				[11189] = { -- Variety is the Spice of Life
					Completed = false,
					Spacing = "      ",
				},
				[10994] = { --A Glorious Campaign
					Completed = false,
					Spacing = "      ",
				},
				[11157] = { --Loremaster of Legion
					Completed = false,
					Spacing = "      ",
					[10059] = { --Ain't No Mountain High Enough
						Completed = false,
						Spacing = "         ",
					},
					[10763] = { -- Azsuna Matata
						Completed = false,
						Spacing = "         ",
					},
					[11124] = { -- Good Suramaritan
						Completed = false,
						Spacing = "         ",
					},
					[19698] = { -- That's Val'sharah Folks!
						Completed = false,
						Spacing = "         ",
					},
					[10790] = { -- Vrykul Story, Bro
						Completed = false,
						Spacing = "         ",
					},
				},
				[11188] = { -- Broken Isles Explorer
					Completed = false,
					Spacing = "      ",
					[10665] = { -- Explore Azsuna
						Completed = false,
						Spacing = "         ",
					},
					[10667] = { -- Explore Highmountain
						Completed = false,
						Spacing = "         ",
					},
					[10669] = { -- Explore Suramar
						Completed = false,
						Spacing = "         ",
					},
					[10668] = { -- Explore Stormheim
						Completed = false,
						Spacing = "         ",
					},
					[10666] = { -- Explore Val'sharah
						Completed = false,
						Spacing = "         ",
					},
				},
			},
			[11544] = { -- Defender of the Broken Isles
				Spacing = "   ",
				Completed = false,
			}, 
			[11543] = { -- Explore Broken Shore
				Completed = false,
				Spacing = "   ",
			}, -- Explore Broken Shore
			[11545] = { -- Legionfall Commander
				Completed = false,
				Spacing = "   ",
			}, 
		},
	}

	Pathfinder.GetColor = function (value)
		value = value * 2;
		local r = (2 - value);
		local g = value;
		local b = 0;
		
		if r > 1 then r = 1 end
		if g > 1 then g = 1 end
		if b > 1 then b = 1 end

		r = string.format("%i", r * 255);
		g = string.format("%i", g * 255);
		b = string.format("%i", b * 255);

		return "ff" .. string.format("%02x", r) .. string.format("%02x", g) .. string.format("%02x", b);
	end

	Pathfinder.Filter = function(id)
		local _, name, _, completed, _, _, _, _, _, icon = GetAchievementInfo(id);
		local quantity = 0
		local required = GetAchievementNumCriteria(id);
		if required > 0 then
			for index = 1, required do
				if select(3, GetAchievementCriteriaInfo(id, index)) == true then
					quantity = quantity + 1;
				end
			end
		end
		return name, completed, icon, quantity, required;
	end


	Pathfinder.Traverse = function(table, key)
		local SubReqs = 0;
		local SubQuan = 0;
		for k, v in pairs(table) do
			if type(v) == "table" then
				table[k] = Pathfinder.Traverse(v, k);
				if table[k].SubReqs and table[k].SubQuan then
					SubReqs = SubReqs + table[k].SubReqs;
					SubQuan = SubQuan + table[k].SubQuan;
				end
			end
		end
		if key then
			table.Name, table.Completed, table.Icon, table.Quantity, table.Required = Pathfinder.Filter(key);
			table.SubReqs = SubReqs + table.Required;
			if table.Completed then
				table.SubQuan = table.SubReqs;
			else
				table.SubQuan = SubQuan + table.Quantity;
			end
		end
		return table;
	end

	Pathfinder.GetPercent = function (n)
		local s;
		if n.Completed then
			s = 1;
		elseif n.Required == 0 then
			s = 0;
		else
			s = 1 * (n.SubQuan / n.SubReqs);
		end
		return "|c" .. Pathfinder.GetColor(s) .. string.format("%#3.2f%%", s * 100) .. "|r";
	end

	Pathfinder.Tooltip = function (frame, item)
		if type(item) == "table" then
			if item.Name then
				if (Pathfinder.Config.ShowCompleted and item.Completed) or not item.Completed then
					frame:AddDoubleLine(item.Spacing .. "|T" .. item.Icon .. ":0|t " .. item.Name, Pathfinder.GetPercent(item));
				end
			end
			for k, v in pairs(item) do
				Pathfinder.Tooltip(frame, v);
			end
		end
	end

	Pathfinder.Update = function()
		Pathfinder.Status = Pathfinder.Traverse(Pathfinder.Status);
		dataobj.text = Pathfinder.GetPercent(Pathfinder.Status[11446]);
	end

	return true;
end

function dataobj:OnTooltipShow()
	self:AddLine("|cffe5cc80Pathfinder - Flying Progression Tracker|r|n|n");
	Pathfinder.Tooltip(self, Pathfinder.Status);
	self:AddLine("|nLeft Click to toggle Achievement Frame");
	self:AddLine("Right Click to show/hide completed achievements")
end

function dataobj:OnEnter()
	GameTooltip:SetOwner(self, "ANCHOR_NONE");
	GameTooltip:SetPoint("TOP", self, "BOTTOM");
	GameTooltip:ClearLines();
	dataobj.OnTooltipShow(GameTooltip);
	GameTooltip:Show();
end

function dataobj:OnLeave()
	GameTooltip:Hide()
end

local function EventHandler(self, event, ...)
	if event == "VARIABLES_LOADED" then
		if Initialize() then 
			initialized = true;
			print("|cffe5cc80Pathfinder v" .. GetAddOnMetadata("Pathfinder", "Version") .. " Loaded!|r");
		end
	end
	if event == "CRITERIA_EARNED" or "PLAYER_ENTERING_WORLD" and initialized then
		Pathfinder.Update();
	end
end

local EventListener = CreateFrame("FRAME", "Pathfinder");
EventListener:RegisterEvent("VARIABLES_LOADED");
EventListener:RegisterEvent("CRITERIA_EARNED");
EventListener:RegisterEvent("PLAYER_ENTERING_WORLD");
EventListener:SetScript("OnEvent", EventHandler);
